//
//  UIColor+YMAdditions.m
//  PCDBank
//
//  Created by Gusont on 2020/12/11.
//  Copyright © 2020 YMBank. All rights reserved.
//

#import "UIColor+YMAdditions.h"

@implementation UIColor (YMAdditions)

+(UIColor *)colorWithHex:(long)hexColor
{
    return [UIColor colorWithHex:hexColor alpha:1.0];
}

+(UIColor *)colorWithHex:(long)hexColor alpha:(float)alpha
{
    int r = (hexColor >> 16) & 0xFF;
    int g = (hexColor >> 8) & 0xFF;
    int b = (hexColor) & 0xFF;
    
    return [UIColor colorWithRed:r / 255.0f
                           green:g / 255.0f
                            blue:b / 255.0f
                           alpha:alpha];
}

+(UIColor *)randomColor
{
    CGFloat red = arc4random_uniform(256)/ 255.0;
    CGFloat green = arc4random_uniform(256)/ 255.0;
    CGFloat blue = arc4random_uniform(256)/ 255.0;
    UIColor *color = [UIColor colorWithRed:red green:green blue:blue alpha:1.0];
    return color;
}

+(UIColor *)colorFormString:(NSString *)string
{
    return [UIColor colorFromHexString:string alpha:1.0];
}

+ (UIColor *)colorFromHexString:(NSString *)title alpha:(CGFloat)alpha {
    // 去空格换行  转大写
    title = [[title stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]] uppercaseString];
    
    // 如果字符串长度小于6,格式不正确,返回透明的颜色
    if (title.length < 6) {
        return [UIColor clearColor];
    }
    
    // 去字符串头
    if ([title hasPrefix:@"0X"]) {
        title = [title substringFromIndex:2];
    }else if ([title hasPrefix:@"#"]) {
        title = [title substringFromIndex:1];
    }
    
    // 去字符串头后如果字符串长度不为6,格式不正确,返回透明颜色
    if (title.length != 6) {
        return [UIColor clearColor];
    }
    
    // 6位16进制转10进制
    NSString *rString = [title substringWithRange:NSMakeRange(0, 2)];
    NSString *gString = [title substringWithRange:NSMakeRange(2, 2)];
    NSString *bString = [title substringWithRange:NSMakeRange(4, 2)];
    
    unsigned int r, g, b;
    [[NSScanner scannerWithString:rString] scanHexInt:&r];
    [[NSScanner scannerWithString:gString] scanHexInt:&g];
    [[NSScanner scannerWithString:bString] scanHexInt:&b];
    
    return [UIColor colorWithRed:r/255.f green:g/255.f blue:b/255.f alpha:alpha];
}

@end
