//
//  YMBarButtonItem.h
//  PCDBank
//
//  Created by Gusont on 2020/12/1.
//  Copyright © 2020 YMBank. All rights reserved.
//

#import "YMBarItem.h"

#define YMBarItemWidth (kSCREEN_WIDTH / 3.0) - 15
/**
 YMBarButtonItem样式

 - YMBarButtonItemPlain: 简单
 - YMBarButtonItemBack: 返回
 */
typedef NS_ENUM(NSUInteger, YMBarButtonItemStyle){

    YMBarButtonItemPlain,

    YMBarButtonItemBack,
};

//NS_ASSUME_NONNULL_BEGIN

@interface YMBarButtonItem : YMBarItem

/**
 *  @brief 快速实例化
 *
 *  @param image  图片
 *  @param style  样式，Plain样式下:存在图片，只显示图片；不存在图片显示标题
 *  @param target 动作源对象
 *  @param action 动作
 *
 *  @return 实例化
 */
- (nullable instancetype)initWithImage:(nullable UIImage *)image style:(YMBarButtonItemStyle)style target:(nullable id)target action:(nullable SEL)action;
/**
 *  @brief 快速实例化
 *
 *  @param title  标题，默认YMBarButtonItemPlain样式
 *  @param target 动作源对象
 *  @param action 动作
 *
 *  @return 实例化
 */
- (nullable instancetype)initWithTitle:(nullable NSString *)title target:(nullable id)target action:(nullable SEL)action;
/**
 *  @brief 快速自定制视图实例化
 *
 *  @param customView 自定制视图，左侧视图左侧对齐，右侧视图右侧对齐
 *
 *  @return 实例化
 */
- (nullable instancetype)initWithCustomView:(nullable UIView *)customView;

/**
 *  @brief 样式
 */
@property(nonatomic,assign) YMBarButtonItemStyle style;

/**
 *  @brief 自定义视图
 */
@property(nullable, nonatomic,strong) UIView *customView;

/**
 *  @brief 动作
 */
@property(nullable, nonatomic) SEL action;

/**
 *  @brief 动作源
 */
@property(nullable, nonatomic,weak) id target;

/**
 设置UIControl的长度 默认导航栏的宽度除以3再减15
 */
@property(nonatomic ,assign) float buttonWidth;


@end

//NS_ASSUME_NONNULL_END
